#!/bin/bash
#
#  Documentation and testing.
#
TMPF=/tmp/abradabraca
#
#  Tokenize the current .config file and remove some redundancy.
#
echo "Generate sparse file"
./config /usr/src/linux/.config token.cfg
#
#  Compress the resulting tokenized file.
#
echo "Compress the file"
gzip <token.cfg > token.gz
ls -la token.gz
#
#  Generate an array from the file contents. In the 'real' world,
#  this would be linked into the kernel constant-data. It is very
#   small.
#
echo "Make an array out of the file"
./makearray token.gz oldconfig.c oldcfg
#                 |            |      |______ Array name
#                 |            |_____________ Source to generate
#                 |__________________________ Input file 
#
#  Here, we will test the new array outside the kernel. It is
#  expected that the real thing will get its data from kmem.
#  Note that both an array was created with the data, plus an
#  unsigned long that contains the array length.
#
echo "Include the array in some 'C' code to test it."
gcc -c -o oldconfig.o oldconfig.c
cat > ${TMPF}.c << EOF
#include <stdio.h>
#include <unistd.h>
#include <sys/fcntl.h>
extern unsigned long oldcfg_len;
extern unsigned char oldcfg[];
int main() {
    int fd;
    fd = open("new.gz", O_RDWR|O_CREAT|O_TRUNC, 0644);
    write(fd, oldcfg, oldcfg_len);
    close(fd);
    return 0;
}
EOF
gcc -Wall -o tester ${TMPF}.c oldconfig.o
rm -f ${TMPF}.c
#
#  Writes a file, new.gz, from the data in the array.
#
echo "Write new data file from the array contents"
./tester
#
#  Expand the file data.
#
echo "Uncompress the new file"
gunzip < new.gz > new.cfg
#
#  Convert to a new .config file, spelled 0.config here.
#
echo "Convert the file back to its original"
./config new.cfg 0.config x
echo "Look at its beginning"
head 0.config
echo "Look at its end"
tail 0.config

