/*
 *  Created 09-FEB-2002      Richard B. Johnson      rjohnson@analogic.com
 *  Released under GPL.
 *
 *  This creates a generic 'C' array with a name you define from
 *  the contents of a file.
 *  Usage:	`makearray infile outfile array-name`.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <string.h>


int main(int args, char *argv[]);

#define BUF_LEN 0x1000
#define ERRORS(s) \
{                 \
    fprintf(stderr, "Error from line %d, file %s, call %s, (%s)\n", \
            __LINE__,__FILE__,(s), strerror(errno));                \
}
#define FAIL -1

int main(int args, char *argv[])
{
    struct stat sb;
    int infd, outfd, stat;
    size_t i, count, len; 
    unsigned char *ibuf, *obuf;
    unsigned char *cp;
    if(args < 4)
    {
        fprintf(stderr, "Usage:\n%s <infile> <outfile> <arrayname>\n", argv[0]);
        exit(EXIT_FAILURE);
    }
    if((infd = open(argv[1], O_RDONLY)) == FAIL)
        ERRORS("open");
    if((outfd = open(argv[2], O_RDWR|O_CREAT|O_TRUNC, 0644 )) == FAIL)
        ERRORS("open");

    if((ibuf = (char *) malloc(BUF_LEN * sizeof(char))) == NULL)
        ERRORS("malloc");
    if((obuf = (char *) malloc(BUF_LEN * sizeof(char) * 8)) == NULL)
        ERRORS("malloc");
    count = 0;

    (void)write(outfd, obuf, sprintf(obuf, "/*\n * Created by %s. Do not "
                                           "modify!\n */\n", argv[0]));
    
    cp = obuf;
    cp += sprintf(obuf, "const unsigned char %s[]={\n\t", argv[3]);
    while((stat = read(infd, ibuf, BUF_LEN)) > 0)
    {
        if((len = cp - obuf) != 0)
            (void)write(outfd, obuf, len);
        cp = obuf;
        for(i=0; i < (size_t) stat; i++)
        {
            cp += sprintf(cp, "0x%02x,", ibuf[i]);
            if(!(++count % 0x08))
                cp += sprintf(cp, "\n\t");
        }
    }
    if((len = cp - obuf) > 2)
        (void)write(outfd, obuf, --len);
    (void)write(outfd, obuf, sprintf(obuf, "};\n"));
    if(fstat(infd, &sb) == FAIL)
        ERRORS("fstat");
    (void)write(outfd, obuf, sprintf(obuf, "const unsigned long %s_len=%ld;\n",
                                            argv[3], sb.st_size));

    (void)close(infd);
    (void)close(outfd);
    free(obuf);
    free(ibuf);
    return 0;
}

